import requests
from .utils import reformat_address , get_API_config

class API :
    def __init__(self):
        (self.zillow_base_url , self.zillow_username , self.zillow_password,
        self.propstream_base_url , self.propstream_username ,
        self.propstream_password , self.propstream_login_url,
        self.tracers_base_url , self.tracers_username,
        self.tracers_password) = get_API_config()

    def get_zillow_output(self,address):
        url = f"{self.zillow_base_url}?address={address}"

        headers = {
            "X-RapidAPI-Host": self.zillow_username,
            "X-RapidAPI-Key": self.zillow_password
        }
        
        response = requests.get(url, headers=headers)
        try :
            data = response.json()
            status = data.get("homeStatus", None)
            broker = data.get("brokerageName" , None)
            price = data.get("price", 0)
            zestimate = data.get("zestimate", 0)
            zestimateLowPercent = data.get("zestimateLowPercent") if data.get("zestimateLowPercent") else 0
            url = data.get("url")
            if zestimate == None or zestimate == 0 :
                zestimate = price
            if broker == "Auction.com" :
                status = "Auction"
            return {
                        "zestimate": zestimate,
                        "bedrooms": data.get("bedrooms", 0),
                        "bathrooms": data.get("bathrooms", 0),
                        "status": status,
                        "living_area": data.get("livingAreaValue", 0),
                        "zillow_link" : "https://www.zillow.com" + str(url) if url else None,
                        "zestimate_low": zestimate*(100-int(zestimateLowPercent))/100 
        }
        except Exception:
            return None
        

    def get_propstream_output(self,address):
        try :
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
                'Referer': 'https://login.propstream.com/'
            }
            

            s = requests.session()
            s.headers = headers
            login_result = s.post(url=self.propstream_login_url, params={"username" : self.propstream_username,"password":self.propstream_password})
            authToken = s.get(url=f"{self.propstream_base_url}eqbackend/resource/auth?t={login_result.url.split('=')[1]}").json().get("authToken")
            s.headers["X-Auth-Token"] = authToken
            
            formatted_address = reformat_address(address)
            suggest = s.get(f"{self.propstream_base_url}eqbackend/resource/auth/ps4/property/suggestionsnew?q={formatted_address}").json()
            property_id = suggest[0].get("id")
            address_type = suggest[0].get("type")
            result = s.get(f'{self.propstream_base_url}eqbackend/resource/auth/ps4/property?id={property_id}&addressType={address_type}').json()
            s.get(f'{self.propstream_base_url}/logout')

            json_array = result["properties"][0]
            owners = [
                {
                "first_name": json_array.get("owner1FirstName", None),
                "middle_name": json_array.get("owner1MiddleName", None),
                "last_name": json_array.get("owner1LastName", None),
                },
                {
                "first_name": json_array.get("owner2FirstName", None),
                "middle_name": json_array.get("owner2MiddleName", None),
                "last_name": json_array.get("owner2LastName", None),   
                }
            ]

            property_data = {
                "address": json_array["address"].get("streetAddress", None),
                "unit": json_array["address"].get("unitNumber", None),
                "city": json_array["address"].get("cityName", None),
                "state": json_array["address"].get("stateCode", None),
                "zip": json_array["address"].get("zip", None),
                "county": json_array["address"].get("countyName", None),
                "mailing": json_array.get("mailAddress",None),
                "apn": json_array.get("apn", None),
                "owner_occupied": json_array.get("ownerOccupied", None),
                "decedent" : json_array.get("decedent", None),
                "occupancy" : json_array.get("ownerProperty",{}).get("vacant", None),
                "property_type" : json_array.get("landUse", None),
                "owners" : owners ,
                "loans": json_array.get("activeLoans", []),
                "liens":json_array.get("liens", []),
                "foreclosures": json_array.get("foreclosure", []),
                "mortgages": json_array.get("mortgages", []),
                "est_remaining_balance": json_array.get("openMortgageBalance", 0),
            }
            
            return property_data
        except :
            return None


    def person_search(self,payload):
        url = self.tracers_base_url
        headers = {
            "accept": "application/json",
            "content-type": "application/json",
            "galaxy-ap-name": self.tracers_username,
            "galaxy-ap-password": self.tracers_password,
            "galaxy-search-type": "BackgroundReport"
        }
        response = requests.post(url, json=payload, headers=headers)
        return response.json()

    def get_tracer_output(self,first_name , middle_name , last_name , address , city , state , zip) :
        payload = {
            "firstName": first_name if first_name == first_name else "",
            "middleName": middle_name if middle_name == middle_name else "",
            "lastName": last_name if last_name == last_name else "",
            "Addresses" : [{
                "AddressLine1" : address,
                "AddressLine2" : f"{state} {zip}"
            }]
        }
        data = self.person_search(payload)["persons"]
        persons = []
        for person in data :
            persons.append({
                "first_name" : person["name"]["firstName"],
                "middle_name" : person["name"]["middleName"],
                "last_name" : person["name"]["lastName"],
                "dob" : person.get("dob", None),
                "age" : person.get("age", None),
                "dod" : person.get("dod", None),
                "addresses" : person.get("addresses", []),
                "associates" : person.get("associates",[]),
                "relatives" : person.get("relatives",[]),
                "emails" : person.get("emailAddress",[]),
                "phones" : person.get("phoneNumbers",[])
                })
            
        return persons


import os
import json
import usaddress
from dotenv import load_dotenv

def get_config() :
    if os.path.exists("config.json"):
        with open("config.json", "r") as file:
            config_file = json.load(file)
            ENV_PATH, LOG_PATH = config_file.get("env_file_path")+"/.env", config_file.get(
                "log_file_path"
            )
            load_dotenv(ENV_PATH)
            return LOG_PATH , os.getenv("BASE_URL") , os.getenv("TOKEN_TIME") , os.getenv("QUEUE_BASE_URL")

def get_API_config() :
    if os.path.exists("config.json"):
        with open("config.json", "r") as file:
            config_file = json.load(file)
            ENV_PATH = config_file.get("env_file_path")+"/.env"
            load_dotenv(ENV_PATH)
            return (os.getenv("ZILLOW_BASE_URL") , os.getenv("ZILLOW_USERNAME") , os.getenv("ZILLOW_PASSWORD"),
                    os.getenv("PROPSTREAM_BASE_URL") , os.getenv("PROPSTREAM_USERNAME") , os.getenv("PROPSTREAM_PASSWORD") , os.getenv("PROPSTREAM_LOGIN_URL") , 
                    os.getenv("TRACERS_BASE_URL") , os.getenv("TRACERS_USERNAME") , os.getenv("TRACERS_PASSWORD"))

def check_address(address):
    parsed_address = usaddress.tag(address)[0]
    check = all(
        [
            parsed_address.get("AddressNumber", None),
            parsed_address.get("StreetName", None),
            parsed_address.get("StateName", None),
            parsed_address.get("ZipCode", None),
        ]
    )
    if not check:
        return None
    else:
        return address


def reformat_address(address):
    try:
        parsed_address = usaddress.tag(address)[0]
        occupancy_identifier = ""
        street_name_post_type = parsed_address.get('StreetNamePostType','')
        if parsed_address.get("OccupancyIdentifier"):
            occupancy_identifier = f"{parsed_address.get('OccupancyIdentifier','').strip()},"
            if not occupancy_identifier.startswith("#"):
                occupancy_identifier = "#"+ occupancy_identifier
            else:
                occupancy_identifier=occupancy_identifier.replace("# ",'#')
        else:
            street_name_post_type +=","

        formatted_address = " ".join(
            [
                parsed_address.get("AddressNumber", ""),
                parsed_address.get("StreetName", ""),
                street_name_post_type,
                occupancy_identifier.strip(),
                parsed_address.get("StateName", ""),
                parsed_address.get("ZipCode", ""),
            ]
        )
        formatted_address = " ".join(formatted_address.split())
        return ", ".join(i.strip() for i in formatted_address.split(","))

    except:
        return None


def save_configs(env_path, log_path):
    env_path = env_path.strip()
    if env_path and os.path.exists(env_path) and log_path and os.path.exists(log_path):
        config = {"env_file_path": env_path, "log_file_path": log_path}
        with open("config.json", "w") as f:
            json.dump(config, f)
        print(f"Saved .env file path and log file path to config.json: {env_path}")
        print(f"Saved .log file path to config.json: {log_path}")
    else:
        raise Exception("Error: Invalid path entered.")


def make_obj(row, columns):
    return {key: row[key] if key in row else None for key in columns}


def check_empty_obj(obj):
    return sum(1 for value in obj.values() if value in (None, 0)) != len(obj)


def make_body(row):
    body = {}

    row = {key.lower().replace(" ", "_"): value for key, value in row.items()}
    row = {key: None if value != value else value for key, value in row.items()}

    columns = ["reformatted_address", "source_name", "is_auction"]
    body.update({"duplicate_check": make_obj(row, columns)})

    columns = [
        "address",
        "county",
        "apn",
        "property_type",
        "lot_size",
        "year_built",
        "occupancy_status",
        "bedroom",
        "bathrooms",
        "zestimate",
        "equity",
        "square_footage",
        "zillow_link",
        "status",
    ]
    body.update({"property": make_obj(row, columns)})

    columns = [
        "first_name",
        "last_name",
        "dob",
        "dod",
        "mailing_address",
        "mailing_city",
        "mailing_state",
        "mailing_zip",
    ]
    owner_obj = make_obj(row, columns)
    if check_empty_obj(owner_obj):
        body.update({"owner": owner_obj})

    columns = [
        "auction_date",
        "estimated_resale_value",
        "opening_bid",
        "estimated_debt",
        "rental_estimate",
        "trustee_sale_number",
        "link",
    ]
    auction_obj = make_obj(row, columns)
    if check_empty_obj(auction_obj):
        body.update({"auction": auction_obj})

    columns = ["sale_date", "sold_amount", "sale_status"]
    salesinfo_obj = make_obj(row, columns)
    if check_empty_obj(salesinfo_obj):
        body.update({"salesInformation": salesinfo_obj})

    columns = [
        "document_name",
        "case_type",
        "total_amount_owed",
        "date_of_filing",
        "plaintiff",
        "plaintiff_attorney_firm",
        "plaintiff_attorney_name",
        "plaintiff_atty_bar_no",
        "defendants",
        "probate_case_number",
    ]
    legalproceeding_obj = make_obj(row, columns)
    if check_empty_obj(legalproceeding_obj):
        body.update({"legalproceeding": legalproceeding_obj})

    columns = [
        "loan_type",
        "interest_rate",
        "lender_name",
        "mortgage_date",
        "mortgage_amount",
        "debt",
    ]
    debtmortgage_obj = make_obj(row, columns)
    if check_empty_obj(debtmortgage_obj):
        body.update({"mortgageanddebt": debtmortgage_obj})

    columns = ["lien_type", "lien_date", "lien_amount", "certificate_of_release"]
    taxlien_obj = make_obj(row, columns)
    if check_empty_obj(taxlien_obj):
        body.update({"taxlien": taxlien_obj})

    connection_obj = []
    for i in range(1, 3):
        addr = row.get(f"associate_{i}_address")
        if addr:
            connection_obj.append(
                {
                    "connection_type": "Associate",
                    "name": row[f"associate_{i}_name"],
                    "address": addr,
                    "phone": row[f"associate_{i}_phone"],
                }
            )

    for j in range(1, 3):
        for i in range(1, 4):
            addr = row.get(f"address_{j}_neighbor_{i}_address")
            if addr:
                connection_obj.append(
                    {
                        "connection_type": "Neighbor",
                        "name": row[f"address_{j}_neighbor_{i}_name"],
                        "address": addr,
                        "phone": row[f"address_{j}_neighbor_{i}_phone"],
                    }
                )

    for i in range(1, 11):
        addr = row.get(f"relative_{i}_address")
        if addr:
            connection_obj.append(
                {
                    "connection_type": "Relative",
                    "name": row[f"relative_{i}_name"],
                    "address": addr,
                    "phone": row[f"relative_{i}_phone"],
                }
            )

    body.update({"connections": connection_obj})

    emails_obj = []
    for i in range(1, 4):
        email = row.get(f"email_{i}")
        if email:
            emails_obj.append({"email_address": email})
    body.update({"emails": emails_obj})

    phones_obj = []
    for i in range(1, 4):
        phone = row.get(f"phone_{i}")
        if phone:
            phones_obj.append(
                {
                    "phone_type": row[f"phone_{i}_type"],
                    "phone_connected": bool(row[f"phone_{i}_is_connected"]),
                    "phone_number": phone,
                }
            )
    body.update({"phones": phones_obj})
    return body



from .API import API
from .utils import make_body, check_address , get_config
import requests
from requests.exceptions import RequestException
import time
import json
import logging

class ProcessLogger:
    def __init__(self, log_file):
        if log_file :
            self.logger = logging.getLogger("ProcessLogger")
            self.logger.setLevel(logging.INFO)
            formatter = logging.Formatter(
                "%(asctime)s | Address: %(address)s | Cause: %(cause)s | Source: %(source)s | Is Auction: %(is_auction)s"
            )
            file_handler = logging.FileHandler(log_file + "/process.log")
            file_handler.setFormatter(formatter)
            self.logger.addHandler(file_handler)

    def log_failure(self, address, cause, source_name, is_auction):
        extra = {
            "address": address,
            "cause": cause,
            "source": source_name,
            "is_auction": is_auction,
        }
        self.logger.info("Processing failure logged.", extra=extra)


class Data:
    def __init__(
        self,
        full_address: str,
        is_auction: bool,
        source_name: str,
        opening_bid: float = None,
        max_debt: float = None,
        auction_value: float = None,
        auction_date: str = None,
        property_type: str = None,
        link: str = None,
        occupancy: str = None,
        trustee_name: str = None,
        trustee_phone: str = None,
        trustee_sale_number: str = None,
        apn: str = None,
        sold_date: str = None,
        sold_value: float = None,
        final_judgment: float = None,
        plaintiff: str = None,
        attorney_name: str = None,
        attorney_phone: str = None,
        attorney_bar_no: str = None,
        attorney_firm: str = None,
        defendants: str = None,
        sales_status: str = None,
        nos_amount: float = None,
        total_amount_owed: float = None,
        document_name: str = None,
        case_number: str = None,
        case_type: str = None,
        date_of_filing: str = None,
        probate_case_number: str = None,
    ):
        """
        Initialize an AuctionProperty instance with the following attributes:

        full_address: Full address of the property.
        is_auction: Boolean indicating if the property is an auction property.
        source_name: Name of the source where the property information is obtained.
        opening_bid: Opening bid amount for the auction.
        max_debt: maximum amount of debt.
        auction_value: highest bid at the auction.
        auction_date: Date of the auction.
        property_type: Type of the property (e.g., residential, commercial).
        link: Link to the property listing or auction details.
        occupancy: Occupancy status of the property.
        trustee_name: Name of the trustee handling the auction.
        trustee_phone: Phone number of the trustee.
        trustee_sale_number: Sale number associated with the trustee.
        apn: Assessor's Parcel Number (APN) of the property.
        sold_date: Date the property was sold.
        sold_value: Value for which the property was sold.
        final_judgment: Final judgment amount.
        plaintiff: Plaintiff in the case associated with the property.
        attorney_name: Name of the attorney handling the case.
        attorney_phone: Phone number of the attorney.
        attorney_bar_no: Bar number of the attorney.
        attorney_firm: Firm the attorney is associated with.
        defendants: Defendants in the case associated with the property.
        sales_status: Sales status of the property.
        nos_amount: Notice of Sale (NOS) amount.
        total_amount_owed: Total amount owed on the property.
        document_name: Name of the document related to the property.
        case_number: Case number associated with the property.
        case_type: Type of the case (e.g., foreclosure, probate).
        date_of_filing: Date the case was filed.
        probate_case_number: Probate case number (if applicable).
        """
        self.full_address = full_address
        self.is_auction = is_auction
        self.source_name = source_name
        self.opening_bid = opening_bid
        self.max_debt = max_debt
        self.auction_value = auction_value
        self.auction_date = auction_date
        self.property_type = property_type
        self.link = link
        self.occupancy = occupancy
        self.trustee_name = trustee_name
        self.trustee_phone = trustee_phone
        self.trustee_sale_number = trustee_sale_number
        self.apn = apn
        self.sold_date = sold_date
        self.sold_value = sold_value
        self.final_judgment = final_judgment
        self.plaintiff = plaintiff
        self.attorney_name = attorney_name
        self.attorney_phone = attorney_phone
        self.attorney_bar_no = attorney_bar_no
        self.attorney_firm = attorney_firm
        self.defendants = defendants
        self.sales_status = sales_status
        self.nos_amount = nos_amount
        self.total_amount_owed = total_amount_owed
        self.document_name = document_name
        self.case_number = case_number
        self.case_type = case_type
        self.date_of_filing = date_of_filing
        self.probate_case_number = probate_case_number
    
    def validate(self,logger):
        """Validate data types of the attributes."""
        validations = {
            'full_address': str,
            'is_auction': bool,
            'source_name': str,
            'opening_bid': (float, type(None)),
            'max_debt': (float, type(None)),
            'auction_value': (float, type(None)),
            'auction_date': (str, type(None)),
            'property_type': (str, type(None)),
            'link': (str, type(None)),
            'occupancy': (str, type(None)),
            'trustee_name': (str, type(None)),
            'trustee_phone': (str, type(None)),
            'trustee_sale_number': (str, type(None)),
            'apn': (str, type(None)),
            'sold_date': (str, type(None)),
            'sold_value': (float, type(None)),
            'final_judgment': (float, type(None)),
            'plaintiff': (str, type(None)),
            'attorney_name': (str, type(None)),
            'attorney_phone': (str, type(None)),
            'attorney_bar_no': (str, type(None)),
            'attorney_firm': (str, type(None)),
            'defendants': (str, type(None)),
            'sales_status': (str, type(None)),
            'nos_amount': (float, type(None)),
            'total_amount_owed': (float, type(None)),
            'document_name': (str, type(None)),
            'case_number': (str, type(None)),
            'case_type': (str, type(None)),
            'date_of_filing': (str, type(None)),
            'probate_case_number': (str, type(None)),
        }
        
        for attr, expected_type in validations.items():
            value = getattr(self, attr)
            if not isinstance(value, expected_type):
                print(f"{attr} must be of type {expected_type}, got {type(value)} instead.")
                logger.log_failure(
                        self.full_address,
                        f"{attr} must be of type {expected_type}, got {type(value)} instead. (bad data format)",
                        self.source_name,
                        self.is_auction,
                )
                return False 
        return True


class AuthManager:
    def __init__(self, username, password):
        _, base_url , token_time , _ = get_config()
        self.base_url = base_url
        self.token_time = token_time
        self.session = None
        self.access_token = None
        self.refresh_token = None
        self.expiration_time = None
        self.username = username
        self.password = password
        self.is_authenticated()

    def get_new_tokens(self):
        
        token_endpoint = f"{self.base_url}/api/token/"
        try:
            self.session = requests.Session()
            response = self.session.post(
                token_endpoint,
                data={"username": self.username, "password": self.password},
            )
            response.raise_for_status()
            tokens = response.json()
            self.access_token = tokens.get("access")
            self.refresh_token = tokens.get("refresh")
            expires_in = tokens.get("expires_in", float(self.token_time))
            self.expiration_time = time.time() + expires_in
        except RequestException:
            self.session = None
            self.access_token = None
            self.refresh_token = None
            self.expiration_time = None
            raise Exception("Failed to authenticate. Check your credentials.")

    def refresh_access_token(self):
        refresh_token_endpoint = f"{self.base_url}/api/token/refresh/"
        try:
            response = self.session.post(
                refresh_token_endpoint, data={"refresh": self.refresh_token}
            )
            response.raise_for_status()
            self.access_token = response.json().get("access")
            self.refresh_token = response.json().get("refresh")
            self.expiration_time = time.time() + float(self.token_time)
        except RequestException:
            self.access_token = None
            self.expiration_time = None
            raise Exception("Failed to refresh token.")

    def get_authenticated_session(self):
        if not self.session or not self.access_token:
            self.get_new_tokens()

        if time.time() >= self.expiration_time:
            try:
                self.refresh_access_token()
            except Exception:
                print("Refresh token expired. Re-authenticating...")
                self.get_new_tokens()

        return self.session

    def is_authenticated(
        self,
    ):
        session = self.get_authenticated_session()
        headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
        }
        response = session.get(self.base_url + "/api/leads", headers=headers)
        if response.status_code == 401:
            raise ("there is a problem with login , check your username and password")

    def make_authenticated_request(self, url, method="GET", data=None):

        session = self.get_authenticated_session()
        headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
        }

        try:
            if method == "GET":
                response = session.get(url, headers=headers)
            elif method == "POST":
                response = session.post(url, headers=headers, data=data)
            else:
                raise ValueError("Unsupported HTTP method")

            if response.status_code == 401:
                print("Token expired, re-authenticating...")
                self.get_new_tokens()
                session = self.get_authenticated_session()

                if method == "GET":
                    response = session.get(url)
                elif method == "POST":
                    response = session.post(url, data=data)

            response.raise_for_status()
            return response.json()
        except RequestException as e:
            print(f"Request failed: {e}")
            return None


class ProcessLogic:
    def __init__(self, DATA):
        LOG_PATH , _ , _ , _ = get_config()
        self.logger = ProcessLogger(LOG_PATH)
        self.DATA = DATA

    def initial_layer(self):
        if not self.DATA.is_auction:
            if all([self.DATA.auction_value, self.DATA.final_judgment]):
                if self.DATA.auction_value - self.DATA.final_judgment < 40000:
                    self.logger.log_failure(
                        self.DATA.full_address,
                        "Auction value and final judgment difference < 40000 (initial layer)",
                        self.DATA.source_name,
                        self.DATA.is_auction,
                    )
                    return False
            elif all([self.DATA.sold_value, self.DATA.opening_bid]):
                if self.DATA.sold_value - self.DATA.opening_bid < 40000:
                    self.logger.log_failure(
                        self.DATA.full_address,
                        "Sold value and opening bid difference < 40000 (initial layer)",
                        self.DATA.source_name,
                        self.DATA.is_auction,
                    )
                    return False
        return self.DATA

    def zillow_layer(self):
        if self.DATA.is_auction:
            exclude_status = [
                "SOLD",
                "OTHERS",
                "RECENTLY_SOLD",
                "FOR_SALE",
                "ACTIVE",
                "PENDING",
                "FOR_RENT",
            ]
            if self.DATA.zestimate_low:
                if self.DATA.status in exclude_status:
                    self.logger.log_failure(
                        address=self.DATA.address,
                        cause=f"Status {self.DATA.status} is in exclude_status list (zillow layer)",
                        source_name=self.DATA.source_name,
                        is_auction=self.DATA.is_auction,
                    )
                    return False
                if self.DATA.zestimate_low <= 150000:
                    self.logger.log_failure(
                        address=self.DATA.address,
                        cause=f"Zestimate low {self.DATA.zestimate_low} <= 150,000 (zillow layer)",
                        source_name=self.DATA.source_name,
                        is_auction=self.DATA.is_auction,
                    )
                    return False
                if self.DATA.opening_bid:
                    if self.DATA.opening_bid >= 0.50 * self.DATA.zestimate_low:
                        self.logger.log_failure(
                            address=self.DATA.address,
                            cause=f"Opening bid {self.DATA.opening_bid} >= 50% of Zestimate low {self.DATA.zestimate_low} (zillow layer)",
                            source_name=self.DATA.source_name,
                            is_auction=self.DATA.is_auction,
                        )
                        return False
                if self.DATA.total_amount_owed:
                    if self.DATA.total_amount_owed >= 0.50 * self.DATA.zestimate_low:
                        self.logger.log_failure(
                            address=self.DATA.address,
                            cause=f"Total amount owed {self.DATA.total_amount_owed} >= 50% of Zestimate low {self.DATA.zestimate_low} (zillow layer)",
                            source_name=self.DATA.source_name,
                            is_auction=self.DATA.is_auction,
                        )
                        return False
                    if self.DATA.zestimate_low - self.DATA.total_amount_owed < 100000:
                        self.logger.log_failure(
                            address=self.DATA.address,
                            cause=f"Difference between Zestimate low and total amount owed "
                            f"({self.DATA.zestimate_low - self.DATA.total_amount_owed}) < 100,000 (zillow layer)",
                            source_name=self.DATA.source_name,
                            is_auction=self.DATA.is_auction,
                        )
                        return False
        return self.DATA

    def propstream_layer(self):

        if self.DATA.max_debt:
            self.DATA.debt_status = "debt provided in website"

        last_name = self.DATA.owners[0].get("last_name")
        if any(
            k.lower() in last_name.lower()
            for k in [
                "Construction",
                "Bank",
                "Developers",
                "Land",
                "Corporation",
                "Holdings",
            ]
        ):
            self.logger.log_failure(
                address=self.DATA.address,
                cause=f"Excluded due to last name '{last_name}' (propstream layer)",
                source_name=self.DATA.source_name,
                is_auction=self.DATA.is_auction,
            )
            return False
        if any(
            k.lower() in self.DATA.property_type.lower()
            for k in [
                "Vacant",
                "Mobile",
                "Rural Residence",
                "Public School",
                "Farm Land",
            ]
        ):
            self.logger.log_failure(
                address=self.DATA.address,
                cause=f"Excluded due to property type '{self.DATA.property_type}' (propstream layer)",
                source_name=self.DATA.source_name,
                is_auction=self.DATA.is_auction,
            )
            return False

        if self.DATA.is_auction:
            if all([self.DATA.opening_bid, self.DATA.final_judgment]):
                if max(self.DATA.opening_bid, self.DATA.final_judgment) > 1000:
                    self.DATA.max_debt = max(
                        self.DATA.opening_bid, self.DATA.final_judgment
                    )
                    self.DATA.debt_status = (
                        "maximum value of opening bid and final judgement"
                    )
            elif (
                any(
                    [
                        self.DATA.est_remaining_balance,
                        self.DATA.loans[0]["estimatedLoanBalance"],
                    ]
                )
                and not self.DATA.max_debt
            ):
                est_balance = (
                    self.DATA.est_remaining_balance
                    if self.DATA.est_remaining_balance is not None
                    else 0
                )
                loan_balance = (
                    self.DATA.loans[0]["estimatedLoanBalance"]
                    if self.DATA.loans[0]["estimatedLoanBalance"] is not None
                    else 0
                )
                self.DATA.max_debt = max(est_balance, loan_balance)
                self.DATA.debt_status = (
                    "maximum value of est remaining balance and loan 1 balance"
                )

            if all([self.DATA.max_debt, self.DATA.zestimate_low]):
                if self.DATA.max_debt >= 0.50 * self.DATA.zestimate_low:
                    self.logger.log_failure(
                        address=self.DATA.address,
                        cause=f"Max debt {self.DATA.max_debt} >= 50% of Zestimate low {self.DATA.zestimate_low} (propstream layer)",
                        source_name=self.DATA.source_name,
                        is_auction=self.DATA.is_auction,
                    )
                    return False
                if self.DATA.zestimate_low - self.DATA.max_debt <= 100000:
                    self.logger.log_failure(
                        address=self.DATA.address,
                        cause=f"Zestimate low - Max debt <= 100,000 (propstream layer)",
                        source_name=self.DATA.source_name,
                        is_auction=self.DATA.is_auction,
                    )
                    return False
        else:
            if all([self.DATA.sold_value, self.DATA.est_remaining_balance]):
                if self.DATA.sold_value - self.DATA.est_remaining_balance < 40000:
                    self.logger.log_failure(
                        address=self.DATA.address,
                        cause=f"Sold value {self.DATA.sold_value} - Estimated remaining balance {self.DATA.est_remaining_balance} < 40,000 (propstream layer)",
                        source_name=self.DATA.source_name,
                        is_auction=self.DATA.is_auction,
                    )
                    return False

        return self.DATA

    def start(self):
        if not self.DATA.validate(self.logger) :
            return False ,"bad data format"
        
        if not check_address(self.DATA.full_address):
            self.logger.log_failure(
                address=self.DATA.full_address,
                cause=f"Address is not valid",
                source_name=self.DATA.source_name,
                is_auction=self.DATA.is_auction,
            )
            return False, "address is not valid"
        
        initial_filter = self.initial_layer()
        if not initial_filter:
            return False, "removed by initial filters"

        api = API()

        ###########Zillow#############
        print(self.DATA.full_address, ": starting Zillow Batching", sep="")
        zillow_output = api.get_zillow_output(self.DATA.full_address)
        if zillow_output:
            for key, value in zillow_output.items():
                setattr(self.DATA, key, value)
            zillow_filter = self.zillow_layer()
            if not zillow_filter:
                return False, "removed by zillow filters"
        else:
            self.logger.log_failure(
                address=self.DATA.address,
                cause=f"Zillow API output is None",
                source_name=self.DATA.source_name,
                is_auction=self.DATA.is_auction,
            )

        ###########Propstream#############
        print(self.DATA.full_address, ": starting Propstream Batching", sep="")
        propstream_output = api.get_propstream_output(self.DATA.full_address)
        if propstream_output:
            for key, value in propstream_output.items():
                setattr(self.DATA, key, value)
            propstream_filter = self.propstream_layer()
            if not propstream_filter:
                return False, "removed by propstream filters"
        else:
            self.logger.log_failure(
                address=self.DATA.address,
                cause=f"Propstream API output is None",
                source_name=self.DATA.source_name,
                is_auction=self.DATA.is_auction,
            )
            return False, "propstream API output is none"
        ###########Tracers#############
        print(self.DATA.full_address, ": starting Tracers Batching", sep="")
        tracers_output = []
        for owner in self.DATA.owners:
            tracers_output += api.get_tracer_output(
                owner.get("first_name"),
                owner.get("middle_name"),
                owner.get("last_name"),
                self.DATA.address,
                self.DATA.city,
                self.DATA.state,
                self.DATA.zip,
            )
        setattr(self.DATA, "persons", tracers_output)
        return self.DATA, "process is done"


class Lead:
    def __init__(self, session):
        _, base_url ,_ , _ = get_config()
        self.base_url = base_url
        self.auth_session = session

    def insert(self, data):
        url = f"{self.base_url}/api/leads/"
        source_name = data.get("source_name", None)
        is_auction = data.get("is_auction", None)
        queue = Queue()
        queue.starting_process()
        completed_data, status = ProcessLogic(data, source_name, is_auction).start()
        queue.process_is_done()
        if completed_data:
            response = self.auth_session.make_authenticated_request(
                url, method="POST", data=json.dumps(make_body(completed_data))
            )
            print(response)
        else:
            print(status)


class Queue:
    def __init__(self):
        _,_,_,queue_base_url = get_config()
        self.queue_base_url = queue_base_url

    def starting_process(self):
        response = requests.get(f"{self.queue_base_url}/process")
        if response.status_code == 200:
            self.request_id = response.json().get("request_id")
            return True
        return False

    def process_is_done(self):
        response = requests.post(
            f"{self.queue_base_url}/stop", json={"request_id": self.request_id}
        )
        if response.status_code == 200:
            return True
        return False



from setuptools import setup, find_packages

setup(
    name='msvproperties',
    version='1.1.6',
    packages=find_packages(),
    license='MIT',
    install_requires=[
        'certifi>=2024.12.14,<2025',
        'charset-normalizer>=3.4.1,<4',
        'idna>=3.10,<4',
        'probableparsing==0.0.1',
        'python-crfsuite==0.9.11',
        'python-dotenv==1.0.1',
        'requests==2.32.3',
        'urllib3==2.3.0',
        'usaddress==0.5.11',
    ],
    python_requires='<4',
    description='A Library for using in our CRM',
    author='Alireza',
    author_email='alireza@msvproperties.net',
    url='https://github.com/alireza-msvproperties/msvproperties/',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.13',
        'License :: OSI Approved :: MIT License',
    ],
)



import warnings
warnings.filterwarnings("ignore")

from .utils import save_configs
from .core import Lead, AuthManager, Data

__all__ = ["Lead", "save_configs", "AuthManager", "Data"]
__version__ = "1.1.6"
__author__ = "Alireza"